// --------------------------------------------------------------------
// Projet POBOT Carte CNP.     		                             			---
// Fonctions Timer. Version 1.0.           													---
//                                                                  ---
// Cr le 30 Janvier 2006 par Spinelli Gilles.                    	---
// --------------------------------------------------------------------


#include "CarteCNP.h"


// ----- Global Variables. -----
volatile unsigned int           iTimer_CptS;		// Variable Timer pour Librairie Srie.
volatile unsigned long					lClock_Second;	// Pour le temps rel.
volatile struct structFlags     sTimer_Flag;

// ----- Privates Constants. -----
#define TIME_MILLISECOND		(255-249)       // Indique que 250x(62.5x64)=1000S est le Temps Millisecondes.
#define TIME_SECOND					1000          	// Indique 1 seconde = 1000 millisecondes.

//#define COLOR_TIMEPERIOD		100							// Indique Changement Couleur toute les 100mS.
#define COLOR_TIMEPERIOD		1000						// Tests : changement couleur toutes les secondes.


// ----- Private Variables. -----
unsigned int          			iTimer_Cpt, iTimer_CptClock;
unsigned int          			iTimer_CptColor;			// Pour les capteurs


// ----------------------------------------------------------------
// ----- Low Level Functions for Timer.                       -----
// ----------------------------------------------------------------

// --- Interruption Timer Millisecondes. Bas sur le OverFlow du Timer 0---
SIGNAL (SIG_OVERFLOW0)
{
  static unsigned char cTemp;
  
  TCNT0 = TCNT0 + TIME_MILLISECOND;                                            
  TIMSK = TIMSK & ~(_BV(TOIE0));      	// Disable Int OVF T0.
  
  enable_interrupt();										// Enable All others interrupts.

  // --- Gestion du Timer Gnral. ---
  if (iTimer_Cpt)
  {
    iTimer_Cpt--;
    if (!iTimer_Cpt)
      { bTimer_FlagCpt = TRUE; }
  }

  // --- Gestion du Timer Serie. Pour dtection fin TimeOut Srie ---
  if (iTimer_CptS)
  {
    iTimer_CptS--;
    if (!iTimer_CptS)
    {
      bTimer_FlagS = TRUE;
      if (bSerial_BeginRec)
      	{ bSerial_EndRec = TRUE; }
    }
  }
  
  // --- Gestion de la Clock Temps Rel. Incrmente Horloge toutes les Secondes ---
  if (iTimer_CptClock)
  {
    iTimer_CptClock--;
    if (!iTimer_CptClock)
    {
      lClock_Second++;
      iTimer_CptClock = TIME_SECOND;
      bTimer_FlagClk = TRUE;
    }
  }
  
  // --- Timer pour Changement Couleur au sol (Pobot 2006) ---
  if (iTimer_CptColor)
  {
  	// -- Teste si la Couleur et si elle a chang. --
  	cTemp = 0;
  	if ((pDetect_G & bDetect_G) == 0)
  		{ cTemp = 1 + bColor_BlueOn; }
  	if (cTemp != cColor_G)
  		{ cColor_G = cTemp; bColor_Change = TRUE; }
  		
  	cTemp = 0;
  	if ((pDetect_D & bDetect_D) == 0)
  		{ cTemp = 1 + bColor_BlueOn; }
  	if (cTemp != cColor_D)
  		{ cColor_D = cTemp; bColor_Change = TRUE; }
  	
  	// -- Teste si Changement Etat Couleur necessaire --
  	iTimer_CptColor--;
  	if (iTimer_CptColor==0)
  	{
  		iTimer_CptColor=COLOR_TIMEPERIOD;
  		bColor_BlueOn = !bColor_BlueOn;
  		if (bColor_BlueOn)
  			{ COLOR_BLUE(); }
  		else
  			{ COLOR_RED(); }
  	}  	
  }
  
	TIMSK = TIMSK | _BV(TOIE0);    					// Enable Int OVF T0.
}


// ----------------------------------------------------------------
// ----- High Level Functions for Timer.                      -----
// ----------------------------------------------------------------

// --- Initialisation Timer Millisecond. ---
void Timer_Init(void)
{
  // -- Initialisation Variables. --
  iTimer_Cpt = 0; bTimer_FlagCpt = FALSE;					// RAZ Timer Gnral
  iTimer_CptS = 0; bTimer_FlagS = FALSE;     			// RAZ Timer Srie.
  iTimer_CptClock = 0; bTimer_FlagClk = FALSE; 		// RAZ CLock Temps Rel.

	// -- Initialisation Variables POBOT 2006. --
  iTimer_CptColor = 0; bColor_BlueOn = FALSE;			// RAZ Timer Color (Pobot 2006).	

  // -- Timer A0 est utilis comme Timer milliseconde. --
  TCCR0 = 0x00;
  TCCR0 = _BV(CS01) | _BV(CS00); 					// Enable Timer T0 avec Clock/64.
  TCNT0 = TIME_MILLISECOND;                                              
  TIFR = _BV(TOV0);        								// Clear Flag OVF T0.
  TIMSK = TIMSK | _BV(TOIE0);    					// Enable Int OVF T0. 
}


// --- Stop Timer Millisecond. ---
void Timer_StandBy(void)
{
	TIMSK = TIMSK & ~(_BV(TOIE0));      	// Disable Int OVF T0.
}


// --- Lauch Timer 'iTime' Millisecond. ---
void Timer(unsigned int iTime)
{
  if (iTime)
  {
    iTimer_Cpt=iTime;
    bTimer_FlagCpt=FALSE;
  }
  else
    { bTimer_FlagCpt=TRUE; }
}


// --- Wait 'iTime' Millisecond. ---
void Delay(unsigned int iTime)
{
  Timer(iTime);
  while (!bTimer_FlagCpt);
}


// --- Wait 'iTime' MicroSecond. ---
void Wait_MicroSecond(unsigned int iTime)
{
  iTime=(iTime/3)*2;

  while (iTime)
  {
    iTime--;
    no_operation();
  }
}


// --- Wait 'iTime' x100 MicroSecond. ---
void Wait_100MicroSecond(unsigned int iTime)
{
  while (iTime)
  {
    iTime--;
    Wait_MicroSecond(98);
  }
}


// ----- Lance l'Horloge Temps Rel -----
void Clock_Init(void)
{
	lClock_Second = 0;
	iTimer_CptClock = TIME_SECOND;
	bTimer_FlagClk = FALSE; 
}


// ----- Donne l'heure au format HH:MM:SS -----
void Clock_Conversion(unsigned char * szBuffer)
{
	unsigned char	cTemp;
	
	cTemp = (unsigned char)((lClock_Second /3600) % 24);// Indique les Heures.			
	szBuffer[0] = (cTemp/10)+'0';
	szBuffer[1] = (cTemp%10)+'0';
	szBuffer[2] = ':';
	
	cTemp = (unsigned char)((lClock_Second /60) % 60);	// Indique les Minutes.			
	szBuffer[3] = (cTemp/10)+'0';
	szBuffer[4] = (cTemp%10)+'0';
	szBuffer[5] = ':';
	
	cTemp = (unsigned char)(lClock_Second % 60);				// Indique les Secondes.
	szBuffer[6] = (cTemp/10)+'0';
	szBuffer[7] = (cTemp%10)+'0';
	szBuffer[8] = 0;
}


// ----- Lance Timer Color (POBOT 2006) -----
void Timer_Color(void)
{
  iTimer_CptColor=COLOR_TIMEPERIOD;
  bColor_BlueOn = TRUE;
  cColor_G = 0; cColor_D = 0;
}
